Exception: NextStation::StepReturnValueError

Inherits:
Error
  • Object
show all
Defined in:
lib/next_station.rb

Overview

Raised when a step method returns something other than a NextStation::State object. This ensures that the Railway flow is maintained throughout the operation.